@@ -98,8 +98,8 @@ impl<T: Mutate> Drop for TxnLockReadGuard<T> {
         let lock = &mut self.lock.inner.lock().unwrap();
         match lock.state.readers.get_mut(&self.txn_id) {
             Some(count) if *count > 1 => (*count) -= 1,
-            Some(1) => {
-                lock.state.readers.remove(&self.txn_id);
+            Some(count) if *count == 1 => {
+                (*count) -= 1;
 
                 while let Some(waker) = lock.state.wakers.pop_front() {
                     waker.wake()
@@ -230,25 +230,16 @@ impl<T: Mutate> TxnLock<T> {
     pub fn try_read(&self, txn_id: &TxnId) -> TCResult<Option<TxnLockReadGuard<T>>> {
         let lock = &mut self.inner.lock().unwrap();
 
-        if !lock.value_at.contains_key(txn_id)
-            && txn_id
-                < lock
-                    .state
-                    .last_commit
-                    .as_ref()
-                    .unwrap_or(&super::id::MIN_ID)
-        {
+        let last_commit = lock.state.last_commit.as_ref().unwrap_or(&super::id::MIN_ID);
+        if !lock.value_at.contains_key(txn_id) && txn_id < last_commit {
             // If the requested time is too old, just return an error.
             // We can't keep track of every historical version here.
+            debug!("transaction {} is already finalized, can't acquire read lock", txn_id);
             Err(TCError::conflict())
         } else if lock.state.reserved.is_some() && txn_id >= lock.state.reserved.as_ref().unwrap() {
-            debug!(
-                "TxnLock {} is already reserved for writing at {}",
-                &self.name,
-                lock.state.reserved.as_ref().unwrap()
-            );
-
             // If a writer can mutate the locked value at the requested time, wait it out.
+            let past_write = lock.state.reserved.as_ref().unwrap();
+            debug!("TxnLock {} is already reserved for writing at {}", &self.name, past_write);
             Ok(None)
         } else {
             // Otherwise, return a ReadGuard.
@@ -382,8 +373,8 @@ impl<T: Mutate> Transact for TxnLock<T> {
 
         let lock = &mut self.inner.lock().unwrap();
 
-        if let Some(readers) = lock.state.readers.get(txn_id) {
-            if readers > &0 {
+        if let Some(readers) = lock.state.readers.remove(txn_id) {
+            if readers > 0 {
                 panic!("tried to finalize a transaction that's still active!")
             }
         }
